@@ -15,6 +15,7 @@
 //! use twilight_cache_inmemory::{InMemoryCache, ResourceType};
 //!
 //! let resource_types = ResourceType::CHANNEL
+//!     | ResourceType::GUILD
 //!     | ResourceType::MEMBER
 //!     | ResourceType::ROLE;
 //!
@@ -29,7 +30,7 @@ use std::{
     fmt::{Display, Formatter, Result as FmtResult},
 };
 use twilight_model::{
-    channel::{permission_overwrite::PermissionOverwrite, GuildChannel},
+    channel::{permission_overwrite::PermissionOverwrite, Channel, ChannelType},
     guild::Permissions,
     id::{
         marker::{ChannelMarker, GuildMarker, RoleMarker, UserMarker},
@@ -83,18 +84,38 @@ impl ChannelError {
 impl Display for ChannelError {
     fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
         match self.kind {
-            ChannelErrorType::ChannelUnavailable { channel_id } => f.write_fmt(format_args!(
-                "channel {} is either not in the cache or is not a guild channel",
-                channel_id
-            )),
-            ChannelErrorType::MemberUnavailable { guild_id, user_id } => f.write_fmt(format_args!(
-                "member (guild: {}; user: {}) is not present in the cache",
-                guild_id, user_id
-            )),
-            ChannelErrorType::RoleUnavailable { role_id } => f.write_fmt(format_args!(
-                "member has role {} but it is not present in the cache",
-                role_id
-            )),
+            ChannelErrorType::ChannelNotInGuild { channel_id } => {
+                f.write_str("channel ")?;
+                Display::fmt(&channel_id, f)?;
+
+                f.write_str(" is not in a guild")
+            }
+            ChannelErrorType::ChannelUnavailable { channel_id } => {
+                f.write_str("channel ")?;
+                Display::fmt(&channel_id, f)?;
+
+                f.write_str(" is either not in the cache or is not a guild channel")
+            }
+            ChannelErrorType::MemberUnavailable { guild_id, user_id } => {
+                f.write_str("member (guild: ")?;
+                Display::fmt(&guild_id, f)?;
+                f.write_str("; user: ")?;
+                Display::fmt(&user_id, f)?;
+
+                f.write_str(") is not present in the cache")
+            }
+            ChannelErrorType::ParentChannelNotPresent { thread_id } => {
+                f.write_str("thread ")?;
+                Display::fmt(&thread_id, f)?;
+
+                f.write_str(" has no parent")
+            }
+            ChannelErrorType::RoleUnavailable { role_id } => {
+                f.write_str("member has role ")?;
+                Display::fmt(&role_id, f)?;
+
+                f.write_str(" but it is not present in the cache")
+            }
         }
     }
 }
@@ -105,6 +126,13 @@ impl Error for ChannelError {}
 #[derive(Debug)]
 #[non_exhaustive]
 pub enum ChannelErrorType {
+    /// Channel is not in a guild.
+    ///
+    /// This may be because the channel is a private channel.
+    ChannelNotInGuild {
+        /// ID of the channel.
+        channel_id: Id<ChannelMarker>,
+    },
     /// Guild channel is not present in the cache.
     ChannelUnavailable {
         /// ID of the channel.
@@ -120,6 +148,11 @@ pub enum ChannelErrorType {
         /// ID of the user.
         user_id: Id<UserMarker>,
     },
+    /// A thread's parent ID is not present.
+    ParentChannelNotPresent {
+        /// ID of the thread.
+        thread_id: Id<ChannelMarker>,
+    },
     /// One of the user's roles is not available in the guild.
     ///
     /// The reasons this could happen could be due to the cache missing a
@@ -177,14 +210,20 @@ impl RootError {
 impl Display for RootError {
     fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
         match self.kind {
-            RootErrorType::MemberUnavailable { guild_id, user_id } => f.write_fmt(format_args!(
-                "member (guild: {}; user: {}) is not present in the cache",
-                guild_id, user_id
-            )),
-            RootErrorType::RoleUnavailable { role_id } => f.write_fmt(format_args!(
-                "member has role {} but it is not present in the cache",
-                role_id
-            )),
+            RootErrorType::MemberUnavailable { guild_id, user_id } => {
+                f.write_str("member (guild: ")?;
+                Display::fmt(&guild_id, f)?;
+                f.write_str("; user: ")?;
+                Display::fmt(&user_id, f)?;
+
+                f.write_str(") is not present in the cache")
+            }
+            RootErrorType::RoleUnavailable { role_id } => {
+                f.write_str("member has role ")?;
+                Display::fmt(&role_id, f)?;
+
+                f.write_str(" but it is not present in the cache")
+            }
         }
     }
 }
@@ -266,6 +305,7 @@ impl<'a> InMemoryCachePermissions<'a> {
     /// The following [`ResourceType`]s must be enabled:
     ///
     /// - [`ResourceType::CHANNEL`]
+    /// - [`ResourceType::GUILD`]
     /// - [`ResourceType::MEMBER`]
     /// - [`ResourceType::ROLE`]
     ///
@@ -306,6 +346,7 @@ impl<'a> InMemoryCachePermissions<'a> {
     ///
     /// [`Permissions::all`]: twilight_model::guild::Permissions::all
     /// [`ResourceType::CHANNEL`]: crate::ResourceType::CHANNEL
+    /// [`ResourceType::GUILD`]: crate::ResourceType::GUILD
     /// [`ResourceType::MEMBER`]: crate::ResourceType::MEMBER
     /// [`ResourceType::ROLE`]: crate::ResourceType::ROLE
     /// [`ResourceType`]: crate::ResourceType
@@ -314,12 +355,15 @@ impl<'a> InMemoryCachePermissions<'a> {
         user_id: Id<UserMarker>,
         channel_id: Id<ChannelMarker>,
     ) -> Result<Permissions, ChannelError> {
-        let channel = self.0.channels_guild.get(&channel_id).ok_or(ChannelError {
+        let channel = self.0.channels.get(&channel_id).ok_or(ChannelError {
             kind: ChannelErrorType::ChannelUnavailable { channel_id },
             source: None,
         })?;
 
-        let guild_id = channel.guild_id();
+        let guild_id = channel.guild_id.ok_or(ChannelError {
+            kind: ChannelErrorType::ChannelNotInGuild { channel_id },
+            source: None,
+        })?;
 
         if self.is_owner(user_id, guild_id) {
             return Ok(Permissions::all());
@@ -329,22 +373,17 @@ impl<'a> InMemoryCachePermissions<'a> {
             .member_roles(user_id, guild_id)
             .map_err(ChannelError::from_member_roles)?;
 
-        let overwrites = match channel.value().resource() {
-            GuildChannel::Category(c) => c.permission_overwrites.clone(),
-            GuildChannel::NewsThread(c) => self.parent_overwrites(&c.id, None)?,
-            GuildChannel::PrivateThread(c) => {
-                self.parent_overwrites(&c.id, Some(c.permission_overwrites.clone()))?
-            }
-            GuildChannel::PublicThread(c) => self.parent_overwrites(&c.id, None)?,
-            GuildChannel::Stage(c) => c.permission_overwrites.clone(),
-            GuildChannel::Text(c) => c.permission_overwrites.clone(),
-            GuildChannel::Voice(c) => c.permission_overwrites.clone(),
+        let overwrites = match channel.kind {
+            ChannelType::GuildPrivateThread
+            | ChannelType::GuildNewsThread
+            | ChannelType::GuildPublicThread => self.parent_overwrites(&channel)?,
+            _ => channel.permission_overwrites.clone().unwrap_or_default(),
         };
 
         let calculator =
             PermissionCalculator::new(guild_id, user_id, everyone, assigned.as_slice());
 
-        Ok(calculator.in_channel(channel.resource().kind(), overwrites.as_slice()))
+        Ok(calculator.in_channel(channel.kind, overwrites.as_slice()))
     }
 
     /// Calculate the guild-level permissions of a member.
@@ -353,6 +392,7 @@ impl<'a> InMemoryCachePermissions<'a> {
     ///
     /// The following [`ResourceType`]s must be enabled:
     ///
+    /// - [`ResourceType::GUILD`]
     /// - [`ResourceType::MEMBER`]
     /// - [`ResourceType::ROLE`]
     ///
@@ -389,6 +429,7 @@ impl<'a> InMemoryCachePermissions<'a> {
     /// member's roles is not in the cache.
     ///
     /// [`Permissions::all`]: twilight_model::guild::Permissions::all
+    /// [`ResourceType::GUILD`]: crate::ResourceType::GUILD
     /// [`ResourceType::MEMBER`]: crate::ResourceType::MEMBER
     /// [`ResourceType::ROLE`]: crate::ResourceType::ROLE
     /// [`ResourceType`]: crate::ResourceType
@@ -470,28 +511,41 @@ impl<'a> InMemoryCachePermissions<'a> {
         }
     }
 
+    /// Given a thread channel, retrieve its parent from the cache, and combine
+    /// parent and child permissions.
     fn parent_overwrites(
         &self,
-        channel_id: &Id<ChannelMarker>,
-        parent_overwrites: Option<Vec<PermissionOverwrite>>,
+        thread: &Channel,
     ) -> Result<Vec<PermissionOverwrite>, ChannelError> {
-        let channel = self.0.channels_guild.get(channel_id).ok_or(ChannelError {
-            kind: ChannelErrorType::ChannelUnavailable {
-                channel_id: *channel_id,
+        let parent_id = thread.parent_id.ok_or(ChannelError {
+            kind: ChannelErrorType::ParentChannelNotPresent {
+                thread_id: thread.id,
             },
             source: None,
         })?;
 
-        if let GuildChannel::Text(c) = channel.resource() {
-            if let Some(parent_overwrites) = parent_overwrites {
-                Ok([c.permission_overwrites.clone(), parent_overwrites.to_vec()].concat())
-            } else {
-                Ok(c.permission_overwrites.clone())
-            }
+        let channel = self.0.channels.get(&parent_id).ok_or(ChannelError {
+            kind: ChannelErrorType::ChannelUnavailable {
+                channel_id: parent_id,
+            },
+            source: None,
+        })?;
+
+        if channel.guild_id.is_some() {
+            let channel_overwrites = channel.permission_overwrites.as_deref().unwrap_or(&[]);
+            let thread_overwrites = thread.permission_overwrites.as_deref().unwrap_or(&[]);
+
+            let mut overwrites =
+                Vec::with_capacity(channel_overwrites.len() + thread_overwrites.len());
+
+            overwrites.extend_from_slice(channel_overwrites);
+            overwrites.extend_from_slice(thread_overwrites);
+
+            Ok(overwrites)
         } else {
             Err(ChannelError {
-                kind: ChannelErrorType::ChannelUnavailable {
-                    channel_id: *channel_id,
+                kind: ChannelErrorType::ChannelNotInGuild {
+                    channel_id: channel.id,
                 },
                 source: None,
             })
@@ -510,11 +564,11 @@ mod tests {
     use twilight_model::{
         channel::{
             permission_overwrite::{PermissionOverwrite, PermissionOverwriteType},
-            Channel, ChannelType, GuildChannel, TextChannel,
+            Channel, ChannelType,
         },
         datetime::Timestamp,
         gateway::payload::incoming::{
-            ChannelCreate, GuildCreate, MemberAdd, MemberUpdate, RoleCreate,
+            ChannelCreate, GuildCreate, MemberAdd, MemberUpdate, RoleCreate, ThreadCreate,
         },
         guild::{
             DefaultMessageNotificationLevel, ExplicitContentFilter, Guild, MfaLevel, NSFWLevel,
@@ -557,6 +611,9 @@ mod tests {
     /// This has the same ID as the [`GUILD_ID`].
     const CHANNEL_ID: Id<ChannelMarker> = GUILD_ID.cast();
 
+    /// ID of a thread created in the general channel.
+    const THREAD_ID: Id<ChannelMarker> = Id::new(5);
+
     fn base_guild() -> Guild {
         Guild {
             id: GUILD_ID,
@@ -617,31 +674,85 @@ mod tests {
     }
 
     fn channel() -> Channel {
-        Channel::Guild(GuildChannel::Text(TextChannel {
+        Channel {
+            application_id: None,
+            bitrate: None,
+            default_auto_archive_duration: None,
             guild_id: Some(GUILD_ID),
+            icon: None,
             id: CHANNEL_ID,
+            invitable: None,
             kind: ChannelType::GuildText,
             last_message_id: None,
             last_pin_timestamp: None,
-            name: "test".to_owned(),
-            nsfw: false,
+            member: None,
+            member_count: None,
+            message_count: None,
+            name: Some("test".to_owned()),
+            newly_created: None,
+            nsfw: Some(false),
+            owner_id: None,
             parent_id: None,
-            permission_overwrites: Vec::from([
+            permission_overwrites: Some(Vec::from([
                 PermissionOverwrite {
                     allow: Permissions::empty(),
                     deny: Permissions::CREATE_INVITE,
-                    kind: PermissionOverwriteType::Role(EVERYONE_ROLE_ID),
+                    id: EVERYONE_ROLE_ID.cast(),
+                    kind: PermissionOverwriteType::Role,
                 },
                 PermissionOverwrite {
                     allow: Permissions::EMBED_LINKS,
                     deny: Permissions::empty(),
-                    kind: PermissionOverwriteType::Member(USER_ID),
+                    id: USER_ID.cast(),
+                    kind: PermissionOverwriteType::Member,
                 },
-            ]),
-            position: 0,
+            ])),
+            position: Some(0),
             rate_limit_per_user: None,
+            recipients: None,
+            rtc_region: None,
+            thread_metadata: None,
             topic: None,
-        }))
+            user_limit: None,
+            video_quality_mode: None,
+        }
+    }
+
+    fn thread() -> Channel {
+        Channel {
+            application_id: None,
+            bitrate: None,
+            default_auto_archive_duration: None,
+            guild_id: Some(GUILD_ID),
+            icon: None,
+            id: THREAD_ID,
+            invitable: None,
+            kind: ChannelType::GuildPublicThread,
+            last_message_id: None,
+            last_pin_timestamp: None,
+            member: None,
+            member_count: None,
+            message_count: None,
+            name: Some("test thread".to_owned()),
+            newly_created: None,
+            nsfw: Some(false),
+            owner_id: None,
+            parent_id: Some(CHANNEL_ID),
+            permission_overwrites: Some(Vec::from([PermissionOverwrite {
+                allow: Permissions::ATTACH_FILES,
+                deny: Permissions::empty(),
+                id: EVERYONE_ROLE_ID.cast(),
+                kind: PermissionOverwriteType::Role,
+            }])),
+            position: Some(0),
+            rate_limit_per_user: None,
+            recipients: None,
+            rtc_region: None,
+            thread_metadata: None,
+            topic: None,
+            user_limit: None,
+            video_quality_mode: None,
+        }
     }
 
     fn role_with_permissions(id: Id<RoleMarker>, permissions: Permissions) -> Role {
@@ -774,6 +885,13 @@ mod tests {
             permissions.in_channel(USER_ID, CHANNEL_ID)?,
         );
 
+        cache.update(&ThreadCreate(thread()));
+
+        assert_eq!(
+            Permissions::EMBED_LINKS | Permissions::SEND_MESSAGES | Permissions::ATTACH_FILES,
+            permissions.in_channel(USER_ID, THREAD_ID)?
+        );
+
         Ok(())
     }
 
